@@ -35,7 +35,7 @@ namespace xt
         using shape_type = std::decay_t<S>;
         using undecay_shape = S;
         using inner_storage_type = FST;
-        using temporary_type = xarray<std::decay_t<typename xexpression_type::value_type>>;
+        using temporary_type = xarray<std::decay_t<typename xexpression_type::value_type>, xexpression_type::static_layout>;
         static constexpr layout_type layout = L;
     };
 
@@ -137,7 +137,7 @@ namespace xt
         using temporary_type = typename xcontainer_inner_types<self_type>::temporary_type;
         using base_index_type = xindex_type_t<shape_type>;
 
-        using simd_value_type = xsimd::simd_type<value_type>;
+        using simd_value_type = xt_simd::simd_type<value_type>;
         using strides_vt = typename strides_type::value_type;
         using slice_type = xtl::variant<detail::xfake_slice<strides_vt>, xkeep_slice<strides_vt>, xdrop_slice<strides_vt>>;
         using slice_vector_type = std::vector<slice_type>;
@@ -204,7 +204,9 @@ namespace xt
         using base_type::storage;
         using base_type::expression;
         using base_type::broadcast_shape;
-        using base_type::has_linear_assign;
+
+        template <class O>
+        bool has_linear_assign(const O& str) const noexcept;
 
         template <class T>
         void fill(const T& value);
@@ -436,6 +438,13 @@ namespace xt
         return base_type::storage()[static_cast<size_type>(offset)];
     }
 
+    template <class CT, class S, layout_type L, class FST>
+    template <class O>
+    inline bool xdynamic_view<CT, S, L, FST>::has_linear_assign(const O&) const noexcept
+    {
+        return false;
+    }
+
     template <class CT, class S, layout_type L, class FST>
     template <class... Args>
     inline auto xdynamic_view<CT, S, L, FST>::unchecked(Args... args) -> reference
