@@ -17,6 +17,7 @@
 #include <vector>
 
 #include <xtl/xiterator_base.hpp>
+#include <xtl/xmeta_utils.hpp>
 #include <xtl/xsequence.hpp>
 
 #include "xexception.hpp"
@@ -110,7 +111,7 @@ namespace xt
         using difference_type = typename subiterator_traits::difference_type;
         using size_type = typename storage_type::size_type;
         using shape_type = typename storage_type::shape_type;
-        using simd_type = xsimd::simd_type<value_type>;
+        using simd_type = xt_simd::simd_type<value_type>;
 
         xstepper() = default;
         xstepper(storage_type* c, subiterator_type it, size_type offset) noexcept;
@@ -128,7 +129,7 @@ namespace xt
         template <class R>
         R step_simd();
 
-        value_type step_leading();
+        void step_leading();
 
         template <class R>
         void store_simd(const R& vec);
@@ -270,24 +271,24 @@ namespace xt
         struct LAYOUT_FORBIDEN_FOR_XITERATOR;
     }
 
-    template <class It, class S, layout_type L>
-    class xiterator : public xtl::xrandom_access_iterator_base<xiterator<It, S, L>,
-                                                               typename It::value_type,
-                                                               typename It::difference_type,
-                                                               typename It::pointer,
-                                                               typename It::reference>,
+    template <class St, class S, layout_type L>
+    class xiterator : public xtl::xrandom_access_iterator_base<xiterator<St, S, L>,
+                                                               typename St::value_type,
+                                                               typename St::difference_type,
+                                                               typename St::pointer,
+                                                               typename St::reference>,
                       private detail::shape_storage<S>
     {
     public:
 
-        using self_type = xiterator<It, S, L>;
+        using self_type = xiterator<St, S, L>;
 
-        using subiterator_type = It;
-        using value_type = typename subiterator_type::value_type;
-        using reference = typename subiterator_type::reference;
-        using pointer = typename subiterator_type::pointer;
-        using difference_type = typename subiterator_type::difference_type;
-        using size_type = typename subiterator_type::size_type;
+        using stepper_type = St;
+        using value_type = typename stepper_type::value_type;
+        using reference = typename stepper_type::reference;
+        using pointer = typename stepper_type::pointer;
+        using difference_type = typename stepper_type::difference_type;
+        using size_type = typename stepper_type::size_type;
         using iterator_category = std::random_access_iterator_tag;
 
         using private_base = detail::shape_storage<S>;
@@ -296,8 +297,9 @@ namespace xt
         using index_type = xindex_type_t<shape_type>;
 
         xiterator() = default;
+
         // end_index means either reverse_iterator && !end or !reverse_iterator && end
-        xiterator(It it, shape_param_type shape, bool end_index);
+        xiterator(St st, shape_param_type shape, bool end_index);
 
         self_type& operator++();
         self_type& operator--();
@@ -315,20 +317,20 @@ namespace xt
 
     private:
 
-        subiterator_type m_it;
+        stepper_type m_st;
         index_type m_index;
         difference_type m_linear_index;
 
         using checking_type = typename detail::LAYOUT_FORBIDEN_FOR_XITERATOR<L>::type;
     };
 
-    template <class It, class S, layout_type L>
-    bool operator==(const xiterator<It, S, L>& lhs,
-                    const xiterator<It, S, L>& rhs);
+    template <class St, class S, layout_type L>
+    bool operator==(const xiterator<St, S, L>& lhs,
+                    const xiterator<St, S, L>& rhs);
 
-    template <class It, class S, layout_type L>
-    bool operator<(const xiterator<It, S, L>& lhs,
-                   const xiterator<It, S, L>& rhs);
+    template <class St, class S, layout_type L>
+    bool operator<(const xiterator<St, S, L>& lhs,
+                   const xiterator<St, S, L>& rhs);
 
     /*********************
      * xbounded_iterator *
@@ -383,9 +385,9 @@ namespace xt
     bool operator<(const xbounded_iterator<It, BIt>& lhs,
                    const xbounded_iterator<It, BIt>& rhs);
 
-    /*******************************
-    * trivial_begin / trivial_end *
-    *******************************/
+    /*****************************
+     * linear_begin / linear_end *
+     *****************************/
 
     namespace detail
     {
@@ -399,54 +401,54 @@ namespace xt
             : std::true_type
         {
         };
+    }
 
-        template <class C>
-        XTENSOR_CONSTEXPR_RETURN auto trivial_begin(C& c) noexcept
+    template <class C>
+    XTENSOR_CONSTEXPR_RETURN auto linear_begin(C& c) noexcept
+    {
+        return xtl::mpl::static_if<detail::has_storage_iterator<C>::value>([&](auto self)
         {
-            return xtl::mpl::static_if<has_storage_iterator<C>::value>([&](auto self)
-            {
-                return self(c).storage_begin();
-            }, /*else*/ [&](auto self)
-            {
-                return self(c).begin();
-            });
-        }
+            return self(c).storage_begin();
+        }, /*else*/ [&](auto self)
+        {
+            return self(c).begin();
+        });
+    }
 
-        template <class C>
-        XTENSOR_CONSTEXPR_RETURN auto trivial_end(C& c) noexcept
+    template <class C>
+    XTENSOR_CONSTEXPR_RETURN auto linear_end(C& c) noexcept
+    {
+        return xtl::mpl::static_if<detail::has_storage_iterator<C>::value>([&](auto self)
         {
-            return xtl::mpl::static_if<has_storage_iterator<C>::value>([&](auto self)
-            {
-                return self(c).storage_end();
-            }, /*else*/ [&](auto self)
-            {
-                return self(c).end();
-            });
-        }
+            return self(c).storage_end();
+        }, /*else*/ [&](auto self)
+        {
+            return self(c).end();
+        });
+    }
 
-        template <class C>
-        XTENSOR_CONSTEXPR_RETURN auto trivial_begin(const C& c) noexcept
+    template <class C>
+    XTENSOR_CONSTEXPR_RETURN auto linear_begin(const C& c) noexcept
+    {
+        return xtl::mpl::static_if<detail::has_storage_iterator<C>::value>([&](auto self)
         {
-            return xtl::mpl::static_if<has_storage_iterator<C>::value>([&](auto self)
-            {
-                return self(c).storage_cbegin();
-            }, /*else*/ [&](auto self)
-            {
-                return self(c).cbegin();
-            });
-        }
+            return self(c).storage_cbegin();
+        }, /*else*/ [&](auto self)
+        {
+            return self(c).cbegin();
+        });
+    }
 
-        template <class C>
-        XTENSOR_CONSTEXPR_RETURN auto trivial_end(const C& c) noexcept
+    template <class C>
+    XTENSOR_CONSTEXPR_RETURN auto linear_end(const C& c) noexcept
+    {
+        return xtl::mpl::static_if<detail::has_storage_iterator<C>::value>([&](auto self)
         {
-            return xtl::mpl::static_if<has_storage_iterator<C>::value>([&](auto self)
-            {
-                return self(c).storage_cend();
-            }, /*else*/ [&](auto self)
-            {
-                return self(c).cend();
-            });
-        }
+            return self(c).storage_cend();
+        }, /*else*/ [&](auto self)
+        {
+            return self(c).cend();
+        });
     }
 
     /***************************
@@ -470,7 +472,7 @@ namespace xt
     {
         if (dim >= m_offset)
         {
-            using strides_value_type = decltype(p_c->strides()[0]);
+            using strides_value_type = typename std::decay_t<decltype(p_c->strides())>::value_type;
             m_it += difference_type(static_cast<strides_value_type>(n) * p_c->strides()[dim - m_offset]);
         }
     }
@@ -480,7 +482,7 @@ namespace xt
     {
         if (dim >= m_offset)
         {
-            using strides_value_type = decltype(p_c->strides()[0]);
+            using strides_value_type = typename std::decay_t<decltype(p_c->strides())>::value_type;
             m_it -= difference_type(static_cast<strides_value_type>(n) * p_c->strides()[dim - m_offset]);
         }
     }
@@ -512,7 +514,7 @@ namespace xt
     template <class C>
     inline void xstepper<C>::to_end(layout_type l)
     {
-        m_it = p_c->data_xend(l);
+        m_it = p_c->data_xend(l, m_offset);
     }
 
     template <class C>
@@ -521,7 +523,7 @@ namespace xt
     {
         R reg;
         reg.load_unaligned(&(*m_it));
-        m_it += xsimd::revert_simd_traits<R>::size;
+        m_it += xt_simd::revert_simd_traits<R>::size;
         return reg;
     }
 
@@ -530,14 +532,13 @@ namespace xt
     inline void xstepper<C>::store_simd(const R& vec)
     {
         vec.store_unaligned(&(*m_it));
-        m_it += xsimd::revert_simd_traits<R>::size;;
+        m_it += xt_simd::revert_simd_traits<R>::size;;
     }
 
     template <class C>
-    auto xstepper<C>::step_leading() -> value_type
+    void xstepper<C>::step_leading()
     {
         ++m_it;
-        return *m_it;
     }
 
     template <>
@@ -868,8 +869,8 @@ namespace xt
     {
         if (end)
         {
-            // Note: the layout here doesn't matter (unused) but using default layout looks more "correct"
-            to_end(XTENSOR_DEFAULT_LAYOUT);
+            // Note: the layout here doesn't matter (unused) but using default traversal looks more "correct".
+            to_end(XTENSOR_DEFAULT_TRAVERSAL);
         }
     }
 
@@ -970,9 +971,9 @@ namespace xt
         };
     }
 
-    template <class It, class S, layout_type L>
-    inline xiterator<It, S, L>::xiterator(It it, shape_param_type shape, bool end_index)
-        : private_base(shape), m_it(it),
+    template <class St, class S, layout_type L>
+    inline xiterator<St, S, L>::xiterator(St st, shape_param_type shape, bool end_index)
+        : private_base(shape), m_st(st),
           m_index(end_index ? xtl::forward_sequence<index_type, const shape_type&>(this->shape())
                             : xtl::make_sequence<index_type>(this->shape().size(), size_type(0))),
           m_linear_index(0)
@@ -991,94 +992,94 @@ namespace xt
         }
     }
 
-    template <class It, class S, layout_type L>
-    inline auto xiterator<It, S, L>::operator++() -> self_type&
+    template <class St, class S, layout_type L>
+    inline auto xiterator<St, S, L>::operator++() -> self_type&
     {
-        stepper_tools<L>::increment_stepper(m_it, m_index, this->shape());
+        stepper_tools<L>::increment_stepper(m_st, m_index, this->shape());
         ++m_linear_index;
         return *this;
     }
 
-    template <class It, class S, layout_type L>
-    inline auto xiterator<It, S, L>::operator--() -> self_type&
+    template <class St, class S, layout_type L>
+    inline auto xiterator<St, S, L>::operator--() -> self_type&
     {
-        stepper_tools<L>::decrement_stepper(m_it, m_index, this->shape());
+        stepper_tools<L>::decrement_stepper(m_st, m_index, this->shape());
         --m_linear_index;
         return *this;
     }
 
-    template <class It, class S, layout_type L>
-    inline auto xiterator<It, S, L>::operator+=(difference_type n) -> self_type&
+    template <class St, class S, layout_type L>
+    inline auto xiterator<St, S, L>::operator+=(difference_type n) -> self_type&
     {
         if (n >= 0)
         {
-            stepper_tools<L>::increment_stepper(m_it, m_index, this->shape(), static_cast<size_type>(n));
+            stepper_tools<L>::increment_stepper(m_st, m_index, this->shape(), static_cast<size_type>(n));
         }
         else
         {
-            stepper_tools<L>::decrement_stepper(m_it, m_index, this->shape(), static_cast<size_type>(-n));
+            stepper_tools<L>::decrement_stepper(m_st, m_index, this->shape(), static_cast<size_type>(-n));
         }
         m_linear_index += n;
         return *this;
     }
 
-    template <class It, class S, layout_type L>
-    inline auto xiterator<It, S, L>::operator-=(difference_type n) -> self_type&
+    template <class St, class S, layout_type L>
+    inline auto xiterator<St, S, L>::operator-=(difference_type n) -> self_type&
     {
         if (n >= 0)
         {
-            stepper_tools<L>::decrement_stepper(m_it, m_index, this->shape(), static_cast<size_type>(n));
+            stepper_tools<L>::decrement_stepper(m_st, m_index, this->shape(), static_cast<size_type>(n));
         }
         else
         {
-            stepper_tools<L>::increment_stepper(m_it, m_index, this->shape(), static_cast<size_type>(-n));
+            stepper_tools<L>::increment_stepper(m_st, m_index, this->shape(), static_cast<size_type>(-n));
         }
         m_linear_index -= n;
         return *this;
     }
 
-    template <class It, class S, layout_type L>
-    inline auto xiterator<It, S, L>::operator-(const self_type& rhs) const -> difference_type
+    template <class St, class S, layout_type L>
+    inline auto xiterator<St, S, L>::operator-(const self_type& rhs) const -> difference_type
     {
         return m_linear_index - rhs.m_linear_index;
     }
 
-    template <class It, class S, layout_type L>
-    inline auto xiterator<It, S, L>::operator*() const -> reference
+    template <class St, class S, layout_type L>
+    inline auto xiterator<St, S, L>::operator*() const -> reference
     {
-        return *m_it;
+        return *m_st;
     }
 
-    template <class It, class S, layout_type L>
-    inline auto xiterator<It, S, L>::operator->() const -> pointer
+    template <class St, class S, layout_type L>
+    inline auto xiterator<St, S, L>::operator->() const -> pointer
     {
-        return &(*m_it);
+        return &(*m_st);
     }
 
-    template <class It, class S, layout_type L>
-    inline bool xiterator<It, S, L>::equal(const xiterator& rhs) const
+    template <class St, class S, layout_type L>
+    inline bool xiterator<St, S, L>::equal(const xiterator& rhs) const
     {
         XTENSOR_ASSERT(this->shape() == rhs.shape());
         return m_linear_index == rhs.m_linear_index;
     }
 
-    template <class It, class S, layout_type L>
-    inline bool xiterator<It, S, L>::less_than(const xiterator& rhs) const
+    template <class St, class S, layout_type L>
+    inline bool xiterator<St, S, L>::less_than(const xiterator& rhs) const
     {
         XTENSOR_ASSERT(this->shape() == rhs.shape());
         return m_linear_index < rhs.m_linear_index;
     }
 
-    template <class It, class S, layout_type L>
-    inline bool operator==(const xiterator<It, S, L>& lhs,
-                           const xiterator<It, S, L>& rhs)
+    template <class St, class S, layout_type L>
+    inline bool operator==(const xiterator<St, S, L>& lhs,
+                           const xiterator<St, S, L>& rhs)
     {
         return lhs.equal(rhs);
     }
 
-    template <class It, class S, layout_type L>
-    bool operator<(const xiterator<It, S, L>& lhs,
-                   const xiterator<It, S, L>& rhs)
+    template <class St, class S, layout_type L>
+    bool operator<(const xiterator<St, S, L>& lhs,
+                   const xiterator<St, S, L>& rhs)
     {
         return lhs.less_than(rhs);
     }
