@@ -93,19 +93,20 @@ namespace xt
         struct conditional_ternary
         {
             template <class B>
-            using get_batch_bool = typename xsimd::simd_traits<typename xsimd::revert_simd_traits<B>::type>::bool_type;
+            using get_batch_bool = typename xt_simd::simd_traits<typename xt_simd::revert_simd_traits<B>::type>::bool_type;
 
-            template <class A1, class A2>
-            constexpr auto operator()(bool t1, const A1& t2, const A2& t3) const noexcept
+            template <class B, class A1, class A2>
+            constexpr auto operator()(const B& cond, const A1& v1, const A2& v2) const noexcept
             {
-                return t1 ? t2 : t3;
+                return xtl::select(cond, v1, v2);
             }
+
             template <class B>
             constexpr B simd_apply(const get_batch_bool<B>& t1,
                                    const B& t2,
                                    const B& t3) const noexcept
             {
-                return xsimd::select(t1, t2, t3);
+                return xt_simd::select(t1, t2, t3);
             }
         };
 
@@ -686,7 +687,7 @@ namespace xt
     /**
      * @ingroup comparison_operators
      * @brief Lesser than
-     * 
+     *
      * Returns an \ref xfunction for the element-wise
      * lesser than comparison of \a e1 and \a e2. This
      * function is equivalent to operator<(E1&&, E2&&).
